<?php


namespace Flat101\LazyLoading\Block\Catalog\Product;


use Magento\Catalog\Block\Product\Image;
use Magento\Catalog\Block\Product\ImageFactory;
use Magento\Catalog\Helper\ImageFactory as HelperFactory;
use Magento\Catalog\Model\Product;

class ImageBuilder extends \Magento\Catalog\Block\Product\ImageBuilder
{
    /**
     * @var \Flat101\LazyLoading\Helper\Data
     */
    private $helperData;

    /**
     * ImageBuilder constructor.
     * @param HelperFactory $helperFactory
     * @param ImageFactory $imageFactory
     */
    public function __construct(
        HelperFactory $helperFactory,
        ImageFactory $imageFactory,
        \Flat101\LazyLoading\Helper\Data $helperData
    ) {
        $this->helperData = $helperData;
        parent::__construct($helperFactory, $imageFactory);
    }

    /**
     * Create image block
     *
     * @param Product|null $product
     * @param string|null $imageId
     * @param array|null $attributes
     * @return Image
     */
    public function create(Product $product = null, string $imageId = null, array $attributes = null)
    {
        $product    = $product ?? $this->product;
        $imageId    = $imageId ?? $this->imageId;
        $attributes = $attributes ?? $this->attributes;
        if (!$this->helperData->isEnabled()) {
            return parent::create($product, $imageId, $attributes);
        }
        $imageFactory           = $this->helperFactory->create()->init($product, $imageId);

        return $this->imageFactory->create($product, $imageId, $attributes);
    }

}
