<?php


namespace Flat101\LazyLoading\Plugin\Cms\Model\Template;


class Filter
{
    /**
     * @var \Flat101\LazyLoading\Helper\Data
     */
    private $helper;

    /**
     * Filter constructor.
     * @param \Flat101\LazyLoading\Helper\Data $helper
     */
    public function __construct(
        \Flat101\LazyLoading\Helper\Data $helper
    ) {
        $this->helper = $helper;
    }

    /**
     * Filter after plugin
     *
     * @param \Magento\Cms\Model\Template\Filter $subject
     * @param string $value
     *
     * @return string
     */
    public function afterFilter(\Magento\Cms\Model\Template\Filter $subject, string $value)
    {
        if (!$this->helper->isEnabled()) {
            return $value;
        }
        preg_match_all('/<img.*?src="(.*?)"[^\>]+>/', $value, $matches);
        $class   = 'lazyload';
        $loading = 'lazy';
        $replace = [];
        $search  = [];
        foreach ($matches[0] as $img) {
            $imgNew = $img;
            //add lazyload attribute
            if (strpos($img, 'loading="') === false) {
                $imgNew = str_replace('<img', '<img loading="' . $loading . '"', $img);
            }
            //add lazyload class
            if (strpos($img, 'class="') !== false) {
                $classes = $this->getElementClasses($img);
                if (!in_array($class, $classes)) {
                    $imgNew = str_replace('class="', 'class="' . $class . ' ', $imgNew);
                }
            } else {
                $imgNew = str_replace('<img', '<img class="' . $class . '"', $imgNew);
            }
            //replace src by data-src
            if (strpos($img, 'data-src="') === false) {
                $imgNew = str_replace('src="', 'data-src="', $imgNew);
            }
            //check if image has exluded classes
            if (!$this->helper->isExcludeClass($this->getElementClasses($imgNew))) {
                $replace[] = $imgNew;
                $search[]  = $img;
            }
        }

        return str_replace($search, $replace, $value);
    }

    /**
     * @param string $element
     *
     * @return array
     */
    public function getElementClasses($element): array
    {
        preg_match('/class\s*=\s*"(.+?)"/', $element, $matches);
        if ($matches) {
            return explode(' ', $matches[1]);
        }

        return [];
    }
}
